@@ -65,7 +65,7 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 /*  Exposed in the read_boulder input routine.... */
 #define PR_NULL_START_CODON_POS       -1000000
-#define PR_DEFAULT_START_CODON_POS    PR_NULL_START_CODON_POS
+#define PR_DEFAULT_START_CODON_POS    -2000000
 #define PR_START_CODON_POS_IS_NULL(SA) ((SA)->start_codon_pos <= PR_NULL_START_CODON_POS)
 
 #define _PR_DEFAULT_POSITION_PENALTIES(PA) \
@@ -425,10 +425,10 @@ typedef struct p3_global_settings {
   double pair_compl_end;
   double pair_compl_end_th;
    
-  int thermodynamic_alignment;
+  int thermodynamic_oligo_alignment;
   /* 
-     Enables to use approach of thermodynamical alignment for dimer,
-     hairpin, and template mispriming calculations.
+     Enables to use approach of thermodynamical alignment for dimer
+     and hairpin calculations.
    
      0 = Use alignment *not* based on thermodynamics - the only dimer
      calculation approach until primer3 2.2.0.  No hairpins are
@@ -436,6 +436,13 @@ typedef struct p3_global_settings {
 
      1 = Use alignment based on thermodynamics. Hairpins are calculated
   */
+  int thermodynamic_template_alignment;
+  /* 
+     Enables to use approach of thermodynamical alignment for template 
+     mispriming calculations.
+     0 = Use alignment *not* based on thermodynamics.
+     1 = Use alignment based on thermodynamics.
+  */
 
   double max_diff_tm; 
   /* Maximum allowed difference between temperature of primer and
@@ -552,6 +559,8 @@ typedef struct primer_rec {
         
   oligo_problems problems;
   char   overlaps_overlap_position;
+
+  char template_mispriming_ok; /* Non-0 if the oligo was checked for this already and it is ok. */
         
 } primer_rec;
 
@@ -966,7 +975,6 @@ void p3_set_gs_primer_self_any_th(p3_global_settings * p , double val);
 void p3_set_gs_primer_self_end(p3_global_settings * p , double val);
 void p3_set_gs_primer_self_end_th(p3_global_settings * p , double val);
 void p3_set_gs_primer_hairpin_th(p3_global_settings * p , double val);
-void p3_set_gs_primer_thermodynamic_alignment(p3_global_settings * p , int val);
 void p3_set_gs_primer_file_flag(p3_global_settings * p , int val);
 void p3_set_gs_primer_pick_anyway(p3_global_settings * p , int val);
 void p3_set_gs_primer_gc_clamp(p3_global_settings * p , int val);
@@ -1018,6 +1026,8 @@ void p3_set_gs_primer_mispriming_library(p3_global_settings * p , char * val);
 void p3_set_gs_primer_internal_oligo_mishyb_library(p3_global_settings * p , char * val);
 void p3_set_gs_primer_max_end_stability(p3_global_settings * p , double val);
 void p3_set_gs_primer_lowercase_masking(p3_global_settings * p , int val);
+void p3_set_gs_primer_thermodynamic_oligo_alignment(p3_global_settings * p , int val);
+void p3_set_gs_primer_thermodynamic_template_alignment(p3_global_settings * p , int val);
 void p3_set_gs_primer_wt_tm_gt(p3_global_settings * p , double val);
 void p3_set_gs_primer_wt_tm_lt(p3_global_settings * p , double val);
 void p3_set_gs_primer_wt_gc_percent_gt(p3_global_settings * p , double val);
@@ -1085,10 +1095,12 @@ args_for_one_oligo_or_primer *p3_get_global_settings_o_args(p3_global_settings *
 int p3_set_afogop_seq_lib(args_for_one_oligo_or_primer *, seq_lib *);
 int p3_set_afogop_opt_tm(args_for_one_oligo_or_primer *, double);
 
+/* max_end_gc must be >= 0 and <= 5 */
+void p3_set_gs_max_end_gc(p3_global_settings *p, int max_end_gc);
+
 void p3_set_gs_max_end_stability(p3_global_settings * p , int max_end_stability);
 void p3_set_gs_gc_clamp(p3_global_settings * p , int gc_clamp);
 void p3_set_gs_lowercase_masking(p3_global_settings * p , int lowercase_masking);
-void p3_set_gs_thermodynamic_alignment(p3_global_settings * p , int thermodynamic_alignment);
 void p3_set_gs_outside_penalty(p3_global_settings * p , double outside_penalty);
 void p3_set_gs_inside_penalty(p3_global_settings * p , double inside_penalty);
 
