@@ -83,16 +83,9 @@ main(int argc, char **argv)
     "                  1  SantaLucia 1998\n"
     "                     *THIS IS THE RECOMMENDED VAULE*\n"
     "                  2  Owczarzy et al., 2004\n\n"
-    "-i             - prints references to publications which were used for thermodynamic calculations\n"
     "\n\n"
     "Prints oligo's melting temperature on stdout.\n";
    
-  const char *info = "1. Breslauer KJ, Frank R, Blcker H and Marky LA. (1986) Predicting DNA duplex stability from the base sequence. Proc. Natl. Acad. Sci., 83, 4746-50.\n\n"
-    "2. Rychlik W, Spencer WJ and Rhoads RE. (1990) Optimization of the annealing temperature for DNA amplification in vitro. Nucleic Acids Res., 118, 6409-12.\n\n"
-    "3. SantaLucia JR. (1998). A unified view of polymer, dumbbell and oligonucleotide DNA nearest-neighbor thermodynamics. Proc. Natl. Acad. Sci., 95, 1460-65.\n\n"
-    "4. Schildkraut, C, and Lifson, S. (1965) Dependence of the melting temperature of DNA on salt concentration. Biopolymers, 3, 195-208.\n\n"
-    "5. Owczarzy R, You Y, Moreira BG, Manthey JA, Huang L, Behlke MA and Walder JA. (2004) Effects of Sodium Ions on DNA Duplex Oligomers: Improved Predictions of Melting Temperatures. Biochemistry, 43, 3537-54.\n";
-   
    const char *copyright = 
 "Copyright (c) 1996,1997,1998,1999,2000,2001,2004,2006\n"
 "Whitehead Institute for Biomedical Research, Steve Rozen\n"
@@ -171,9 +164,6 @@ main(int argc, char **argv)
 	 exit(-1);
        }
        i++;
-     } else if (!strncmp("-i", argv[i], 2)) {
-       fprintf(stderr, info, argv[0]);
-       exit(-1);	    
      } else if (!strncmp("-", argv[i], 1)) {
        /* Unknown option. */
        fprintf(stderr, msg, argv[0]);
