@@ -61,10 +61,6 @@
 #define THAL_EXIT_ON_ERROR 0
 #endif
 
-#ifndef THAL_MAX_SEQ
-#define THAL_MAX_SEQ   10000
-#endif
-
 /* table where bp-s enthalpies, that retrieve to the most stable Tm, are saved */
 #ifdef EnthalpyDPT
 # undef EnthalpyDPT
@@ -95,6 +91,10 @@
 #define atPenaltyS(a, b) atpS[a][b]
 #define atPenaltyH(a, b) atpH[a][b]
 
+#define STR(X) #X
+#define LONG_SEQ_ERR_STR(MAX_LEN) "Target sequence length > maximum allowed (" STR(MAX_LEN) ") in thermodynamic alignment"
+#define XSTR(X) STR(X)
+
 #define SMALL_NON_ZERO 0.000001
 #define DBL_EQ(X,Y) (((X) - (Y)) < (SMALL_NON_ZERO) ? (1) : (2)) /* 1 when numbers are equal */
 
@@ -108,8 +108,12 @@
 # ifdef INTEGER
 const double _INFINITY = 999999.0;
 # else
+# ifdef INFINITY
+const double _INFINITY = INFINITY;
+# else
 const double _INFINITY = 1.0 / 0.0;
 # endif
+# endif
 
 static const double R = 1.9872; /* cal/Kmol */
 static const double ILAS = (-300 / 310.15); /* Internal Loop Entropy ASymmetry correction -0.3kcal/mol*/
@@ -377,7 +381,7 @@ thal(const unsigned char *oligo_f,
    int *bp;
    unsigned char *oligo2_rev = NULL;
    double mh, ms;
-   
+
    send5 = hend5 = NULL;
    enthalpyDPT = entropyDPT = NULL;
    numSeq1 = numSeq2 = NULL;
@@ -401,11 +405,12 @@ thal(const unsigned char *oligo_f,
    /* The following error messages will be seen by end users and will
       not be easy to understand. */
    CHECK_ERROR((len_f > THAL_MAX_ALIGN) && (len_r > THAL_MAX_ALIGN),
-	       "Sequences longer than THAL_MAX_ALIGN for thermodynamical alignment (nearest-neighbor approach)");
+	       "Both sequences longer than " XSTR(THAL_MAX_ALIGN)
+               " for thermodynamic alignment");
    CHECK_ERROR((len_f > THAL_MAX_SEQ), 
-	       "Sequence 1 longer than THAL_MAX_SEQ and alignment is requested");
+	       LONG_SEQ_ERR_STR(THAL_MAX_SEQ) " (1)");
    CHECK_ERROR((len_r > THAL_MAX_SEQ), 
-	       "Sequence 2 longer than THAL_MAX_SEQ and alignment is requested");
+	       LONG_SEQ_ERR_STR(THAL_MAX_SEQ) " (2)");
 
    CHECK_ERROR(NULL == a,  "NULL 'in' pointer");
    if (NULL == o) return; /* Leave it to the caller to crash */
@@ -633,6 +638,24 @@ set_thal_default_args(thal_args *a)
    a->dimer = 1; /* by default dimer structure is calculated */
 }
 
+/* Set default args for oligo */
+void
+set_thal_oligo_default_args(thal_args *a)    
+{
+   memset(a, 0, sizeof(*a));
+   a->debug = 0;
+   a->type = thal_any; /* thal_alignment_type THAL_ANY */
+   a->maxLoop = MAX_LOOP;
+   a->mv = 50; /* mM */
+   a->dv = 0.0; /* mM */
+   a->dntp = 0.0; /* mM */
+   a->dna_conc = 50; /* nM */
+   a->temp = 310.15; /* Kelvin */
+   a->temponly = 1; /* return only melting temperature of predicted structure */
+   a->dimer = 1; /* by default dimer structure is calculated */
+}
+
+
 static unsigned char 
 str2int(char c)
 {
@@ -756,7 +779,7 @@ static double
 saltCorrectS (double mv, double dv, double dntp)
 {
    if(dv<=0) dntp=dv;
-   return 0.368*((log((mv+120*(sqrt(dv-dntp)))/1000)));
+   return 0.368*((log((mv+120*(sqrt(fmax(0.0, dv-dntp))))/1000)));
 }
 
 
@@ -1261,6 +1284,7 @@ fillMatrix(int maxLoop, thal_results *o)
 {
    int d, i, j, ii, jj;
    double* SH;
+
    SH = (double*) safe_malloc(2 * sizeof(double), o);
    for (i = 1; i <= len1; ++i) {
       for (j = 1; j <= len2; ++j) {
@@ -2358,8 +2382,8 @@ symmetry_thermo(const unsigned char* seq)
    seq_end--;
    while(i<mp) {
       i++;
-      s=*seq;
-      e=*seq_end;
+      s=toupper(*seq);
+      e=toupper(*seq_end);
       if ((s=='A' && e!='T')
 	  || (s=='T' && e!='A')
 	  || (e=='A' && s!='T')
