@@ -74,9 +74,10 @@ sub main() {
     # select STDERR;
     $| = 1;
 
+    print "$0 for primer3_core version 2.3.5\n";
+    
     $all_ok = 1;
-    my $start_time;
-    $start_time = time();
+    my $overall_start_time = time();
 
     if (defined $Config{sig_name}) {
           my $i = 0;
@@ -94,7 +95,7 @@ sub main() {
                     'windows',
                     'fast',
                     'verbose',
-		    'onetest=s',
+                    'onetest=s',
                     'executable=s',
                     )) {
         print "Usage: perl p3test.pl \\\n",
@@ -165,7 +166,8 @@ sub main() {
 			    'primer_must_use_th' => 1,
 			    'primer_new_tasks_th' => 1,
 			    'primer_task_th' => 1,
-
+	                    'primer_thal_args' => 1,
+	
 			    # 'test_compl_error' => 1,
 			    # 'test_left_to_right_of_right' => 1,
 
@@ -174,6 +176,11 @@ sub main() {
 			    # 'primer_boundary1' => 1,
 			    # 'primer_boundary_formatted' => 1,
 			    # 'primer_boundary1_formatted' => 1,
+ 
+                            'primer3_v1_1_4_default_settings',
+                            'primer3web_v0_4_0_default_settings',
+   	                    'primer3web_v3_0_0_default_settings',
+                            'primer3web_v4_0_0_default_settings',
 			    
 			    # 'primer_internal' => 1,
 			    # 'primer_internal1' => 1,
@@ -237,85 +244,92 @@ sub main() {
 			    );
 
     my @TESTS = ( 
-		  'th-w-other-tasks',
+			# New tests that use new melting temperature
+			# and thermodynamic alignments
+			# Put the quickest tests first.
+			'primer_must_use_th',
+			'primer_task_th',
+			'primer_thal_args',
+	                'primer_thal_max_seq_error',
+	                'primer_first_base_index',
 
-	          'primer_thermod_align',             
-	          'primer_thermod_align_formatted',
+			'test_compl_error',
+			'test_left_to_right_of_right',
+                        'dv_conc_vs_dntp_conc',
+			'primer_boundary',
+			'primer_boundary1',
+			'primer_boundary_formatted',
+			'primer_boundary1_formatted',
 
-		  # New tests that use new melting temperature
-		  # and thermodynamic alignments
-		  'primer1_th',
-		  'primer_mispriming_th',
-		  'primer_must_use_th',
-		  'primer_new_tasks_th',
-		  'primer_task_th',
+                        'primer3_v1_1_4_default_settings',
+                        'primer3web_v0_4_0_default_settings',
+	                'primer3web_v3_0_0_default_settings',
+                        'primer3web_v4_0_0_default_settings',
 
+			'primer_internal',
+			'primer_internal1',
+			'primer_internal_formatted',
+			'primer_internal1_formatted',
 
-		  'test_compl_error',
-		  'test_left_to_right_of_right',
-	          'primer_boundary', # Put the quickest tests first.
-                  'primer_boundary1',
-                  'primer_boundary_formatted',
-                  'primer_boundary1_formatted',
+			'primer_ok_regions',
+			'primer_ok_regions_formatted',
+			'primer_ok_regions2',
 
-                  'primer_internal',
-                  'primer_internal1',
-                  'primer_internal_formatted',
-                  'primer_internal1_formatted',
+			'primer_tm_lc_masking',
+			'primer_tm_lc_masking_formatted',
 
-	          'primer_ok_regions',
-	          'primer_ok_regions_formatted',
-                  'primer_ok_regions2',
+			'primer_start_codon',
 
-                  'primer_tm_lc_masking',
-                  'primer_tm_lc_masking_formatted',
+			'primer_task',
+			'primer_task_formatted',
+			'primer_renewed_tasks',
+			'primer_new_tasks',
+			'primer_new_tasks_formatted',
 
-                  'primer_start_codon',
+			'primer_must_overlap_point',
+			'primer_overlap_junction',
 
-                  'primer_task',
-                  'primer_task_formatted',
-                  'primer_renewed_tasks',
-                  'primer_new_tasks',
-                  'primer_new_tasks_formatted',
+			'primer_all_settingsfiles',
+			'primer_high_tm_load_set',
+			'primer_high_gc_load_set',
 
-                  'primer_must_overlap_point',
-	          'primer_overlap_junction',
+			'primer_gc_end',
+			'primer_check',
+			'primer_must_use',
+			'primer_must_use_formatted',
+			'primer_syntax',
+			'primer_end_pathology',
+			'primer_num_best',
+			'primer_quality_boundary',
+			'primer',
+			'primer1',
+			'primer_mispriming',
+			'primer_mispriming_formatted',
+			'primer_mispriming_boundary1',
+			'primer_mispriming_boundary1_formatted',
+			'primer_mispriming_boundary2',
+			'primer_mispriming_boundary2_formatted',
+			'primer_mispriming_long_lib',
+			'primer_rat',
+			'primer_human',
+			'long_seq',
+			'primer_position_penalty',
+			'primer_position_penalty_formatted',
+			'p3-tmpl-mispriming',
 
-                  'primer_all_settingsfiles',
-                  'primer_high_tm_load_set',
-                  'primer_high_gc_load_set',
+			'primer_three_prime_distance',
 
-                  'primer_gc_end',
-                  'primer_check',
-                  'primer_must_use',
-                  'primer_must_use_formatted',
-                  'primer_syntax',
-                  'primer_end_pathology',
-                  'primer_num_best',
-                  'primer_quality_boundary',
-                  'primer',
-                  'primer1',
-                  'primer_mispriming',
-                  'primer_mispriming_formatted',
-                  'primer_mispriming_boundary1',
-                  'primer_mispriming_boundary1_formatted',
-                  'primer_mispriming_boundary2',
-                  'primer_mispriming_boundary2_formatted',
-                  'primer_mispriming_long_lib',
-                  'primer_rat',
-                  'primer_human',
-                  'long_seq',
-                  'primer_position_penalty',
-                  'primer_position_penalty_formatted',
-                  'p3-tmpl-mispriming',
-                  # Put slow tests last
+			'primer_obj_fn',
+			'p3_3_prime_n',
 
-                  'p3_3_prime_n',
-	          'primer_three_prime_distance',
-
-                  'primer_obj_fn',
-
-                  'primer_lib_amb_codes',
+			# Put slow tests last
+			'primer_mispriming_th',
+			'th-w-other-tasks',
+			'primer_new_tasks_th',
+			'primer_thermod_align',             
+			'primer_thermod_align_formatted',
+			'primer1_th',
+			'primer_lib_amb_codes',
 
 		  );
 
@@ -332,9 +346,9 @@ sub main() {
 
         # We are inside the for loop here....
         print "$test...";
+	my $test_start_time = time();
 
-        if ($fastFlag && (($test eq 'p3_3_prime_n')
-            || ($test eq 'th-w-other-tasks')
+        if ($fastFlag && (($test eq 'th-w-other-tasks')
             || ($test eq 'primer_obj_fn')
             || ($test eq 'primer1_th')
             || ($test eq 'primer_mispriming_th')
@@ -422,7 +436,11 @@ sub main() {
             # back to main directory
             if (!chdir "../") { die "chdir \"..\": $!\n" }
 
-        } elsif ($test =~ /formatted$/) {
+        } elsif ($test =~ /settings$/) {
+	    my $cmd = "$valgrind_prefix$exe $default_version -strict_tags -p3_settings_file=../$test.txt -echo <$input >$tmp";
+            $r = _nowarn_system($cmd);
+
+	} elsif ($test =~ /formatted$/) {
             my $cmd = "$valgrind_prefix$exe $default_version -strict_tags -format_output <$input >$tmp";
             $r = _nowarn_system($cmd);
 
@@ -443,8 +461,11 @@ sub main() {
         }
 
         $r = perldiff $output, $tmp;
+	print ((time() - $test_start_time), "s ");
+
         if ($r == 0) {
             print "[OK]\n";
+
         } else {
             $all_ok = 0;
             print "[FAILED]\n";
@@ -475,8 +496,7 @@ sub main() {
             print $test. "_list_files...";
             if ($r == 0) {
                 print "[OK]\n";
-            } 
-            else {
+            } else {
                 $all_ok = 0;
                 print "[FAILED]\n";
                 $exit_stat = -1;
@@ -510,7 +530,7 @@ sub main() {
             $exit_stat = -1;
         }
     }
-    print "\nTests ran for ", (time() - $start_time), " seconds.\n";
+    print "\nTests in $0 ran for ", (time() - $overall_start_time), " s\n";
     print "\n\nDONE ", scalar(localtime), " ";
 
     print $all_ok ? "Passed all tests - [OK]\n\n\n" : "At least one test failed - [FAILED]\n\n\n";
@@ -529,7 +549,9 @@ sub perldiff($$) {
     my @f2 = <F2>;
     # If different number of lines, return FAIL.
     if (@f1 != @f2) {
-        print "Different number of lines\n";
+        print "\n   Different number of lines;\n";
+	print "   on command line in test directory run:\n";
+	print "   diff $f1 $f2\n\n";
         return 1;
     }
     # check for differences on the lines, themselves
@@ -594,7 +616,12 @@ sub perldiff($$) {
         # Check for difference between two edited lines (line by line)
         if ($l1 ne $l2) {
             print 
-                "Difference found at line $linenumber:\n<  $l1_orig\n>  $l2_orig\n";
+                "\n   Difference found at line $linenumber:\n",
+		"   <  $l1_orig\n",
+		"   >  $l2_orig\n";
+	    print "   There may be additional differences.\n";
+	    print "   On command line in test directory run\n\n";
+	    print "   diff $f1 $f2\n\n";
             return 1;
         }
     }
@@ -657,7 +684,7 @@ sub test_fatal_errors() {
     if ($fatal_error_problem == 1){
         $all_ok = 0;
     }
-    print $fatal_error_problem ? "[FAILED]" : "[OK]" ,"\n";
+    print(($fatal_error_problem ? "[FAILED]" : "[OK]") ,"\n");
 }
 
 sub _nowarn_system($) {
